package com.example.greenwatch.repositories;


import androidx.lifecycle.MutableLiveData;

import com.example.greenwatch.models.Device;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

public class DeviceRepository {

    private String delimiter = ", ";
    private String deviceUUID;
    private static DeviceRepository deviceRepositoryInstance;
    private MutableLiveData<List<Device>> deviceList = new MutableLiveData<>();
    private HashMap<String, Device> connectedDevicesList = new HashMap<>();
    private HashMap<String, String> deviceIDMapper = new HashMap<>();

    public static synchronized DeviceRepository getInstance() {
        if (deviceRepositoryInstance == null){
            deviceRepositoryInstance = new DeviceRepository();
            deviceRepositoryInstance.setDeviceUUID();
        }
        return deviceRepositoryInstance;
    }

    public MutableLiveData<List<Device>> getConnectedDeviceList() {
        setMutableLiveDataValue();
        return deviceList;
    }

    public void createNewDevice(String timeStamp, String deviceID, boolean sensorStatus, String sensorType, int sensorMassage){
        Device newDevice = new Device(timeStamp, deviceID, sensorStatus, sensorType, sensorMassage);
        addToConnectedDeviceList(newDevice.getDeviceID(), newDevice);
        setMutableLiveDataValue();
    }

    public void createNewRemoteDevice(String message) {
        String[] messageString = messageStringSplitter(message);
        String timeStamp = messageString[0];
        String deviceID = messageString[1];
        boolean sensorStatus = Boolean.valueOf(messageString[1]);
        String sensorType = messageString[3];
        int sensorMassage = Integer.valueOf(messageString[4]);

        if(!connectedDevicesList.containsKey(deviceID)) {
            createNewDevice(timeStamp, deviceID, sensorStatus, sensorType, sensorMassage);
        }
    }

    public String messageStringBuilder(String deviceID) {
        StringBuilder message = new StringBuilder();
        Device device = connectedDevicesList.get(deviceID);
        if(device != null) {
            message.append(device.getTimeStamp())
                    .append(delimiter)
                    .append(device.getDeviceID())
                    .append(delimiter)
                    .append(device.getSensorStatus())
                    .append(delimiter)
                    .append(device.getSensorType())
                    .append(delimiter)
                    .append(device.getSensorMassage());
            return message.toString();
        }
        return message.toString();
    }

    public String getLocalDeviceUUID() {
        return deviceUUID;
    }

    public void setDevice(String deviceID, String timeStamp, String newDeviceID, boolean sensorStatus, String sensorType, int sensorMassage) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            device.setTimeStamp(timeStamp);
            device.setDeviceID(newDeviceID);
            device.setSensorStatus(sensorStatus);
            device.setSensorType(sensorType);
            device.setSensorMassage(sensorMassage);
            setDeviceIDMapper(newDeviceID);
            connectedDevicesList.remove(checkedDeviceID);
            addToConnectedDeviceList(newDeviceID, device);
            setMutableLiveDataValue();
        }
    }

    public void setTimeStamp(String deviceID, String timeStamp) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            device.setTimeStamp(timeStamp);
            addToConnectedDeviceList(checkedDeviceID, device);
            setMutableLiveDataValue();
        }
    }

    public String getTimeStamp(String deviceID) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            return device.getTimeStamp();
        }
        return "";
    }

    public void setDeviceID(String deviceID, String newDeviceID) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            device.setDeviceID(newDeviceID);
            setDeviceIDMapper(newDeviceID);
            connectedDevicesList.remove(checkedDeviceID);
            addToConnectedDeviceList(newDeviceID, device);
            setMutableLiveDataValue();
        }
    }

    public String getDeviceID(String deviceID) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            return device.getDeviceID();
        }
        return "";
    }

    public void setSensorStatus(String deviceID, boolean sensorStatus) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            device.setSensorStatus(sensorStatus);
            addToConnectedDeviceList(checkedDeviceID, device);
            setMutableLiveDataValue();
        }
    }

    public boolean getSensorStatus(String deviceID) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            return device.getSensorStatus();
        }
        return false;
    }

    public void setSensorType(String deviceID, String sensorType) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            device.setSensorType(sensorType);
            addToConnectedDeviceList(checkedDeviceID, device);
            setMutableLiveDataValue();
        }
    }

    public String getSensorType(String deviceID) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            return device.getSensorType();
        }
        return "";
    }

    public void setSensorMassage(String deviceID, int sensorMessage) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            device.setSensorMassage(sensorMessage);
            addToConnectedDeviceList(checkedDeviceID, device);
            setMutableLiveDataValue();
        }
    }

    public int getSensorMassage(String deviceID) {
        String checkedDeviceID = checkDeviceID(deviceID);
        Device device = connectedDevicesList.get(checkedDeviceID);
        if(device != null) {
            return device.getSensorMassage();
        }
        return 0;
    }

    private void setMutableLiveDataValue() {
        List<Device> list = new ArrayList<>(connectedDevicesList.values());
        deviceList.setValue(list);
    }

    private void addToConnectedDeviceList(String key, Device device) {
        connectedDevicesList.put(key, device);
    }

    private String checkDeviceID(String deviceID) {
        if(!deviceIDMapper.isEmpty() && deviceID.equals(deviceUUID)) {
            return deviceIDMapper.get(deviceID);
        }
        return deviceID;
    }

    private void setDeviceIDMapper(String deviceID) {
        deviceIDMapper.put(deviceUUID, deviceID);
    }

    private void setDeviceUUID(){
        this.deviceUUID = UUID.randomUUID().toString();
    }

    private String[] messageStringSplitter(String message) {
        return message.split(delimiter);
    }
}
